************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* Perils of Unobserved Heterogeneity;
* Scope and Limitations of Panel Data Analysis
************************************************************************

/*
This do-file generates the scatter plots in the sections "Heterogeneity" and "Dynamic Models", supported by regression estimates.

See the presentation slides for details on the data-generating process for the artificial data.
See the Stata help files for details on command syntax and functionality.

Note: The default graph color style changed in Stata 18.
With earlier versions, this do-file does not exactly replicate the appearance of the graphs in the presentation slides.
*/

*----------------------------------------------------------------------*
* general graph options
local twoway_options	`"ylabel(-5(1)5) xlabel(-5(1)5) ytitle("y") xtitle("x") legend(off)"'
local scatter_options	"msize(vsmall)"
local color_options		"colorvar(a) colorlist(stc4 stc5 stc6) colordiscrete clegend(off)"
// local graph				"nodraw"

*----------------------------------------------------------------------*
* general set up
clear all
set seed 20230717							// random-number seed
set obs 3000								// number of observations
egen int id = seq(), from(1) block(5)		// unit identifier
egen byte t = seq(), from(1) to(5)			// time identifier
xtset id t									// declare data set to be panel data

*----------------------------------------------------------------------*
* scenario 1: x uncorrelated with a
gen x = rnormal()							// regressor
gen a = mod(id - 1, 3) - 1					// unit-specific error component
gen y = a + rnormal()						// outcome variable

twoway scatter y x, `scatter_options' || lfit y x, lcolor(black) || ,		/// graph specification
	`twoway_options' name(motivation1) `graph'								// graph options

if c(stata_version) >= 18 {
	twoway scatter y x, `scatter_options' `color_options' ||																/// scatter plot
		lfit y x if a == -1, lcolor(stc4) || lfit y x if a == 0, lcolor(stc5) || lfit y x if a == 1, lcolor(stc6) || ,		/// fitted-line graphs for the 3 groups
		`twoway_options' name(motivation2) `graph'																			// graph options
}
else {
	twoway scatter y x if a == -1, `scatter_options' || scatter y x if a == 0, `scatter_options' || scatter y x if a == 1, `scatter_options' ||		/// scatter plots for the 3 groups
		lfit y x if a == -1 || lfit y x if a == 0 || lfit y x if a == 1 || ,																		/// fitted-line graphs for the 3 groups
		`twoway_options' name(motivation2) `graph'																									// graph options
}

reg y x, vce(robust)						// pooled OLS estimator with robust standard errors
xtreg y x, re								// "random-effects" estimator

*----------------------------------------------------------------------*
* scenario 2: x correlated with a
replace x = x + a

if c(stata_version) >= 18 {
	twoway scatter y x, `scatter_options' `color_options' ||																/// scatter plot
		lfit y x if a == -1, lcolor(stc4) || lfit y x if a == 0, lcolor(stc5) || lfit y x if a == 1, lcolor(stc6) || ,		/// fitted-line graphs for the 3 groups
		`twoway_options' name(motivation3) `graph'																			// graph options
}
else {
	twoway scatter y x if a == -1, `scatter_options' || scatter y x if a == 0, `scatter_options' || scatter y x if a == 1, `scatter_options' ||		/// scatter plots for the 3 groups
		lfit y x if a == -1 || lfit y x if a == 0 || lfit y x if a == 1 || ,																		/// fitted-line graphs for the 3 groups
		`twoway_options' name(motivation3) `graph'																									// graph options
}

twoway scatter y x, `scatter_options' || lfit y x, lcolor(black) || ,		/// graph specification
	`twoway_options' name(motivation4) `graph'								// graph options

reg y x, vce(robust)						// pooled OLS estimator with robust standard errors
xtreg y x, re								// "random-effects" estimator

by id: egen y_M = mean(y)
by id: egen x_M = mean(x)
gen y_DM = y - y_M							// outcome variable in deviations from unit-specific means
gen x_DM = x - x_M							// regressor in deviations from unit-specific means

if c(stata_version) >= 18 {
	twoway scatter y_DM x_DM, `scatter_options' `color_options' ||																				/// scatter plot
		lfit y_DM x_DM if a == -1, lcolor(stc4) || lfit y_DM x_DM if a == 0, lcolor(stc5) || lfit y_DM x_DM if a == 1, lcolor(stc6) || ,		/// fitted-line graphs for the 3 groups
		`twoway_options' ytitle("y - mean(y)") xtitle("x - mean(x)") name(motivation5) `graph'													// graph options
}
else {
	twoway scatter y_DM x_DM if a == -1, `scatter_options' || scatter y_DM x_DM if a == 0, `scatter_options' || scatter y_DM x_DM if a == 1, `scatter_options' ||		/// scatter plots for the 3 groups
		lfit y_DM x_DM if a == -1 || lfit y_DM x_DM if a == 0 || lfit y_DM x_DM if a == 1 || ,																			/// fitted-line graphs for the 3 groups
		`twoway_options' ytitle("y - mean(y)") xtitle("x - mean(x)") name(motivation5) `graph'																			// graph options
}

xtreg y x, fe								// "fixed-effects" estimator

*----------------------------------------------------------------------*
* scenario 3: lagged dependent variable
replace x = L.y								// lagged dependent variable

if c(stata_version) >= 18 {
	twoway scatter y x, `scatter_options' `color_options' ||																							/// scatter plot
		lfit y x if a == -1, lcolor(stc4) || lfit y x if a == 0, lcolor(stc5) || lfit y x if a == 1, lcolor(stc6) || lfit y x, lcolor(black) || ,		/// fitted-line graphs for the 3 groups
		`twoway_options' xtitle("L.y") name(motivation6) `graph'																						// graph options
}
else {
	twoway scatter y x if a == -1, `scatter_options' || scatter y x if a == 0, `scatter_options' || scatter y x if a == 1, `scatter_options' ||		/// scatter plots for the 3 groups
		lfit y x if a == -1 || lfit y x if a == 0 || lfit y x if a == 1 || lfit y x, lcolor(black) || ,												/// fitted-line graphs for the 3 groups
		`twoway_options' xtitle("L.y") name(motivation6) `graph'																					// graph options
}

reg y x, vce(robust)						// pooled OLS estimator with robust standard errors

drop x_M x_DM
by id: egen x_M = mean(x)
gen x_DM = x - x_M							// lagged dependent variable in deviations from unit-specific means

if c(stata_version) >= 18 {
	twoway scatter y_DM x_DM, `scatter_options' `color_options' ||																				/// scatter plot
		lfit y_DM x_DM if a == -1, lcolor(stc4) || lfit y_DM x_DM if a == 0, lcolor(stc5) || lfit y_DM x_DM if a == 1, lcolor(stc6) || ,		/// fitted-line graphs for the 3 groups
		`twoway_options' ytitle("y - mean(y)") xtitle("L.y - mean(L.y)") name(motivation7) `graph'												// graph options
}
else {
	twoway scatter y_DM x_DM if a == -1, `scatter_options' || scatter y_DM x_DM if a == 0, `scatter_options' || scatter y_DM x_DM if a == 1, `scatter_options' ||		/// scatter plots for the 3 groups
		lfit y_DM x_DM if a == -1 || lfit y_DM x_DM if a == 0 || lfit y_DM x_DM if a == 1 || ,																			/// fitted-line graphs for the 3 groups
		`twoway_options' ytitle("y - mean(y)") xtitle("L.y - mean(L.y)") name(motivation7) `graph'																		// graph options
}

xtreg y x, fe								// "fixed-effects" estimator
