************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* Perils of Unobserved Heterogeneity;
* Scope and Limitations of Panel Data Analysis
************************************************************************

/*
This do-file generates the simulated estimator distributions in the section "Dynamic Models".

See the presentation slides for details on the data-generating process for the simulated data.
See the Stata help files for details on command syntax and functionality.

Note: The default graph color style changed in Stata 18.
With earlier versions, this do-file does not exactly replicate the appearance of the graphs in the presentation slides.
*/

*----------------------------------------------------------------------*
* general set up
clear all
set seed 20230717							// random-number seed

*----------------------------------------------------------------------*
* simulation program
program define mysimul, rclass
	local T = `1'							// number of time-series observations
	local Tall = `T' + 10					// number of time-series observations (including "burn-in" periods)
	local N = 3000 / `T'					// number of cross-sectional observations
	local lambda = 0.2						// coefficient of the lagged dependent variable
	local beta = 1							// coefficient of the independent regressor

	drop _all
	local obs = `N' * (`Tall')
	set obs `obs'									// number of observations (including "burn-in" periods)
	egen int id = seq(), from(1) block(`Tall')		// unit identifier
	egen byte t = seq(), from(1) to(`Tall')			// time identifier
	xtset id t										// declare data set to be panel data

	gen x = rnormal()						// regressor
	gen a = mod(id - 1, 3) - 1				// unit-specific error component
	gen y = (`beta' * x + a) / (1 - `lambda') + rnormal() / sqrt(1 - (`lambda')^2) if t == 1		// outcome variable (initial observations drawn from the stationary distribution, conditional on x and a)
	replace y = `lambda' * L.y + `beta' * x + a + rnormal() if t > 1								// outcome variable (recursive process for subsequent observations)
	drop if t <= `Tall' - `T'				// drop "burn-in" periods

	xtreg y x, fe							// static "fixed-effects" estimator
	local b_x = _b[x]

	ereturn clear
	return scalar b_x = `b_x'				// return results
end

*----------------------------------------------------------------------*
* simulation with T = 5 (and N = 600)
simulate , reps(10000) dots(10) : mysimul 5		// 10000 replications
sum b_x											// summary statistics
save course1_simulation1, replace

histogram b_x, normal name(simulation1)																									/// graph specification
	ylabel(0(5)25, nolabel noticks) yscale(off) ytitle("")																				/// y-axis options
	xline(1 1.25) xlabel(0.85(0.05)1.3, format("%3.2f")) xmlabel(1 "{&beta}" 1.25 "{&beta}/(1-{&lambda})", labgap(*10)) xtitle("")		// x-axis options

*----------------------------------------------------------------------*
* simulation with T = 50 (and N = 60)
simulate , reps(10000) dots(10) : mysimul 50		// 10000 replications
sum b_x												// summary statistics
save course1_simulation2, replace

histogram b_x, normal name(simulation2)																									/// graph specification
	ylabel(0(5)25, nolabel noticks) yscale(off) ytitle("")																				/// y-axis options
	xline(1 1.25) xlabel(0.85(0.05)1.3, format("%3.2f")) xmlabel(1 "{&beta}" 1.25 "{&beta}/(1-{&lambda})", labgap(*10)) xtitle("")		// x-axis options
