************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* Methods to Analyze a Simple Dynamic Panel Data Model
************************************************************************

/*
This do-file generates the graphs in the section "Bias Correction".

See the presentation slides for details on the data-generating process.
See the Stata help files for details on command syntax and functionality.

Note: The default graph color style changed in Stata 18.
With earlier versions, this do-file does not exactly replicate the appearance of the graphs in the presentation slides.
*/

*----------------------------------------------------------------------*
* general set up
local lambda = 0.2							// coefficient of the lagged dependent variable

*----------------------------------------------------------------------*
* inconsistency of the "fixed-effects" estimator
clear all
set obs 50
gen byte T = _n								// number of time-series observations
gen plim = (1 - (1 - (`lambda')^(T-1)) / ((1 - `lambda') * (T-1)))
replace plim = `lambda' - (1 + `lambda') / (T-2) * plim / (1 - 2 * (`lambda') / ((1 - `lambda') * (T-2)) * plim)		// probability limit of the "fixed-effects" estimator for values of T <= 50
forvalues t = 3 / 50 {
	local plim`t' = plim[`t']
}

twoway line plim T, name(inconsisteny)																			/// graph specification
	yline(`lambda') ylabel(-0.4(0.1)1, format(%2.1f)) ymlabel(`lambda' "{&lambda}", labgap(*15)) ytitle("")		/// y-axis options
	xlabel(0(5)50) xmtick(##5, nolabel)																			// x-axis options

*----------------------------------------------------------------------*
* (finite-sample) bias of the "fixed-effects" estimator
drop _all
set seed 20230717							// random-number seed

program define mysimul, rclass
	local T = `1'							// number of time-series observations
	local Tall = `T' + 10					// number of time-series observations (including "burn-in" periods)
	local N = 3000 / `T'					// number of cross-sectional observations
	local lambda = `2'						// coefficient of the lagged dependent variable

	drop _all
	local obs = `N' * (`Tall')
	set obs `obs'									// number of observations (including "burn-in" periods)
	egen byte id = seq(), from(1) block(`Tall')		// unit identifier
	egen byte t = seq(), from(1) to(`Tall')			// time identifier
	xtset id t										// declare data set to be panel data

	gen a = mod(id - 1, 3) - 1				// unit-specific error component
	gen y = a / (1 - `lambda') + rnormal() / sqrt(1 - (`lambda')^2) if t == 1		// outcome variable (initial observations drawn from the stationary distribution, conditional on x and a)
	replace y = `lambda' * L.y + a + rnormal() if t > 1								// outcome variable (recursive process for subsequent observations)
	drop if t <= `Tall' - `T'				// drop "burn-in" periods

	xtreg y L.y, fe							// "fixed-effects" estimator
	local b_Ly = _b[L.y]

	ereturn clear
	return scalar b_Ly = `b_Ly'				// return results
end

simulate , reps(10000) dots(10) : mysimul 5 `lambda'		// simulation with T = 5 (and N = 600); 10000 replications
sum b_Ly													// summary statistics
local plim : display %4.3f round(`plim5', 0.001)
save course3_bias, replace

histogram b_Ly, normal name(bias)																													/// graph specification
	ylabel(0(5)25, nolabel noticks) yscale(off) ytitle("")																							/// y-axis options
	xline(`plim5' `lambda') xlabel(-0.4(0.1)1, format("%2.1f")) xmlabel(`plim5' "plim = `plim'" `lambda' "{&lambda}", labgap(*10)) xtitle("")		// x-axis options

*----------------------------------------------------------------------*
* iterative bias correction
drop _all
set obs 51
gen lambda = `plim5' in 1					// value of the initial estimator
gen j = _n - 1
local T = 5									// number of time-series observations

quietly forvalues j = 2 / 51 {				// 
	local bT = (1 - (1 - (lambda[`j'-1])^(`T'-1)) / ((1 - lambda[`j'-1]) * (`T'-1)))
	local bT = - (1 + lambda[`j'-1]) / (`T'-2) * `bT' / (1 - 2 * (lambda[`j'-1]) / ((1 - lambda[`j'-1]) * (`T' - 2)) * `bT')		// bias expression
	replace lambda = `plim5' - `bT' in `j'				// iterative bias correction step (the ``fixed-effects'' estimator is simply set to its probability limit for illustrative purposes)
	if abs(lambda[`j'] - lambda[`j'-1]) < 1e-7 {		// convergence condition
		continue, break
	}
}

twoway line lambda j if !missing(lambda), name(iterative)														/// graph specification
	yline(`lambda') ylabel(-0.4(0.1)1, format(%2.1f)) ymlabel(`lambda' "{&lambda}", labgap(*15)) ytitle("")		/// y-axis options
	xlabel(0(5)20) xmtick(##5, nolabel) xtitle("iteration step")												// x-axis options
