************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* Methods to Analyze a Simple Dynamic Panel Data Model
************************************************************************

/*
This do-file generates the line plot of objective functions in the section "Estimation Methods".

See the presentation slides for details on the data-generating process.
See the Stata help files for details on command syntax and functionality.

Note: The default graph color style changed in Stata 18.
With earlier versions, this do-file does not exactly replicate the appearance of the graphs in the presentation slides.
*/

*----------------------------------------------------------------------*
* general set up
clear all
set seed 20230717							// random-number seed
local lambda = 0.2							// coefficient of the lagged dependent variable
local N = 60000								// number of cross-sectional observations (large number of units chosen to approximate asymptotic limit)
local T = 5									// number of time-series observations
local obs = `N' * `T'
set obs `obs'								// number of observations
egen long id = seq(), from(1) block(`T')	// unit identifier
egen byte t = seq(), from(1) to(`T')		// time identifier
xtset id t									// declare data set to be panel data

*----------------------------------------------------------------------*
* data generation
gen a = mod(id - 1, 3) - 1					// unit-specific error component
gen y = a / (1 - `lambda') + rnormal() / sqrt(1 - (`lambda')^2) if t == 1		// outcome variable (initial observations)
replace y = `lambda' * L.y + a + rnormal() if t > 1								// outcome variable (recursive process for subsequent observations)
gen Ly = L.y								// lagged dependent variable
by id: egen y_M = mean(y) if t > 1
by id: egen Ly_M = mean(Ly) if t > 1
gen y_DM = y - y_M							// outcome variable in deviations from unit-specific means
gen Ly_DM = Ly - Ly_M						// lagged dependent variable in deviations from unit-specific means

*----------------------------------------------------------------------*
* objective functions
gen OLS = .
gen ML = .
gen MM = .
quietly forvalues l = 1 / 199 {				// loop over candidate estimates for lambda in the open interval (-1, 1) in steps of 0.01
	gen e = (y_DM - (`l' / 100 - 1) * Ly_DM)
	gen e2 = e^2
	sum e2, mean
	replace OLS = r(mean) in `l'			// value of the OLS objective function
	local s2 = r(mean) * (`T'-1) / (`T'-2)
	replace ML = - ln(2 * c(pi) * `s2') - 2 - r(mean) / (2 * `s2') in `l'		// value of the ML objective function
	gen m = Ly_DM * e
	sum m, mean
	replace MM = (r(mean))^2 in `l'			// value of the MM objective function
	drop e e2 m
}

*----------------------------------------------------------------------*
* graph
local plim = (1 - (1 - (`lambda')^(`T'-1)) / ((1 - `lambda') * (`T'-1)))
local plim = - (1 + `lambda') / (`T'-2) * `plim' / (1 - 2 * (`lambda') / ((1 - `lambda') * (`T'-2)) * `plim')
local plim = `lambda' + `plim'
local plim : display %4.3f `plim'			// probability limit of the "fixed-effects" estimator
gen lambda = _n / 100 - 1 in 1 / 199

twoway line OLS lambda, yaxis(1) || line MM lambda, yaxis(1) || line ML lambda, yaxis(2) || , legend(order(- "left axis:" 1 2 - "" - "right axis:" 3))		/// graph specification
	ylabel(0(0.4)1.6, format("%2.1f") axis(1)) ylabel(-5(0.25)-4, format("%3.2f") axis(2)) ytitle("", axis(2))												/// y-axis options
	xline(`plim' `lambda') xlabel(-1(0.2)1, format("%2.1f")) xmlabel(`plim' "plim = `plim'" `lambda' "{&lambda}", labgap(*10)) xtitle("")					// x-axis options
