************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* Methods to Analyze a Simple Dynamic Panel Data Model
************************************************************************

/*
This do-file generates the line plots of objective functions in the section "ML Estimation".

See the presentation slides for details on the data-generating process.
See the Stata help files for details on command syntax and functionality.

Note: The default graph color style changed in Stata 18.
With earlier versions, this do-file does not exactly replicate the appearance of the graphs in the presentation slides.
*/

*----------------------------------------------------------------------*
* required additional Stata packages
net install xtdpdbc, from("http://www.kripfganz.de/stata/")			// package to compute the BC-MM estimator
net install xtdpdqml, from("http://www.kripfganz.de/stata/")		// package to compute the QML estimator

*----------------------------------------------------------------------*
* general set up
clear all
set seed 20230717							// random-number seed
local lambda = 0.2							// coefficient of the lagged dependent variable
local N = 6000								// number of cross-sectional observations
local T = 5									// number of time-series observations
local obs = `N' * `T'
set obs `obs'								// number of observations
egen byte id = seq(), from(1) block(`T')	// unit identifier
egen byte t = seq(), from(1) to(`T')		// time identifier
xtset id t									// declare data set to be panel data

*----------------------------------------------------------------------*
* data generation
gen a = mod(id - 1, 3) - 1					// unit-specific error component
gen y = a / (1 - `lambda') + rnormal() / sqrt(1 - (`lambda')^2) if t == 1		// outcome variable (initial observations)
replace y = `lambda' * L.y + a + rnormal() if t > 1								// outcome variable (recursive process for subsequent observations)

*----------------------------------------------------------------------*
* objective functions
gen BCMM = .
gen QML = .
quietly forvalues l = 1 / 25 {				// loop over candidate estimates for lambda in the interval [0, 1.2] in steps of 0.05
	noisily display `l'
	local lambda = (`l' - 1) / 20
	xtdpdbc y, noconstant from(`lambda', copy) iterate(0)		// BC-MM estimator, evaluated at the candidate estimate
	replace BCMM = e(f) in `l'				// value of the BC-MM objective function
	xtdpdqml y, noconstant from(`lambda', copy) iterate(0)		// QML estimator, evaluated at the candidate estimate
	replace QML = e(ll) / e(N) in `l'		// value of the QML objective function
	mata: mata drop xtdpdbc_opt_1			// free up memory from a large Mata object
}

*----------------------------------------------------------------------*
* graph
gen lambda = (_n - 1) / 20 in 1 / 25

twoway line BCMM lambda, yaxis(1) || line QML lambda, yaxis(2) || , legend(order(- "left axis:" 1 - "" - "right axis:" 2))						/// graph specification
	ylabel(0(0.03)0.18, format("%3.2f") axis(1)) ylabel(-1.64(0.01)-1.58, format("%3.2f") axis(2)) ytitle("", axis(1)) ytitle("", axis(2))		/// y-axis options
	xline(`lambda') xlabel(0(0.1)1.2, format("%2.1f")) xmlabel(`lambda' "{&lambda}", labgap(*10)) xtitle("")									// x-axis options
