clear all
set seed 20230717							// random-number seed
local reps = 1001							// number of replications
local IV "L2D.y"							// instruments "L2D.y", "L2.y", or "L2D.y L2.y"
local Vcons = 0								// 0: variance of y depends on lambda; 1: variance of y is constant irrespective of lambda
local SDa = 1								// standard deviation of the unit-specific error component

program define mysimul, rclass
	local T = `1'							// number of time-series observations
	local Tall = `T' + 10					// number of time-series observations (including "burn-in" periods)
	local N = 3000 / `T'					// number of cross-sectional observations
	local lambda = `2'						// coefficient of the lagged dependent variable

	drop _all
	local obs = `N' * (`Tall')
	set obs `obs'									// number of observations (including "burn-in" periods)
	egen byte id = seq(), from(1) block(`Tall')		// unit identifier
	egen byte t = seq(), from(1) to(`Tall')			// time identifier
	xtset id t										// declare data set to be panel data

	gen a = `3' * mod(id - 1, 3) - 1				// unit-specific error component
	gen e = rnormal()								// idiosyncratic error component
	if `4' {
		replace a = (1 - `lambda') * a
		replace e = sqrt(1 - (`lambda')^2) * e
	}
	gen y = a / (1 - `lambda') + e / sqrt(1 - (`lambda')^2) if t == 1		// outcome variable (initial observations drawn from the stationary distribution, conditional on x and a)
	replace y = `lambda' * L.y + a + e if t > 1								// outcome variable (recursive process for subsequent observations)
	drop if t <= `Tall' - `T'						// drop "burn-in" periods

	ivregress 2sls D.y (LD.y = `5'), noconstant		// Anderson-Hsiao IV estimator
	local b_Ly = _b[LD.y]

	ereturn clear
	return scalar b_Ly = `b_Ly'
end

frame create simul mean p05 p50 p95
forvalues l = 1 / 20 {
	local lambda = (`l' - 1) / 20
	simulate , reps(`reps') dots(10) : mysimul 5 `lambda' `SDa' `Vcons' "`IV'"		// simulation with T = 5 (and N = 600)
	gen int rep = _n
	quietly replace b_Ly = b_Ly - `lambda'
	egen mean = mean(b_Ly)
	egen p05 = pctile(b_Ly), p(5)
	egen p50 = pctile(b_Ly), p(50)
	egen p95 = pctile(b_Ly), p(95)
	frame post simul (mean[1]) (p05[1]) (p50[1]) (p95[1])
}
frame change simul

set obs 21
gen lambda = (_n - 1) / 20
if "`IV'" == "L2D.y" {
	if `Vcons' {
		gen cov = - (1 - lambda)^2
	}
	else {
		gen cov = - (1 - lambda) / (1 + lambda)
	}
	gen pi = - (1 - lambda) / 2
}
else if "`IV'" == "L2.y" {
	if `Vcons' {
		gen cov = - (1 - lambda)
		gen pi = - (1 - lambda) * (1 / (`SDa'^2 + 1))
	}
	else {
		gen cov = - 1 / (1 + lambda)
		gen pi = - (1 - lambda)^2 / ((1 + lambda) * `SDa'^2 + 1 - lambda)
	}
}
else {
	gen cov = .
	gen pi = .
}

twoway rarea p05 p95 lambda if p05 > -2 & p95 < 2, lwidth(none) fintensity(15) || scatter mean lambda, msymbol(Oh) || scatter p50 lambda, msymbol(+) ||		///
	line cov lambda , yaxis(2) || line pi lambda , yaxis(2) || ,																							/// graph specification
	legend(order(- "left axis:" 1 "5% quantile range" 2 "mean" 3 "median" - "" - "right axis:" 4 "Cov{sub:zx}" 5 "{&pi}=Cov{sub:zx}/Var{sub:z}"))			/// graph options
	ylabel(-2(0.5)2, format("%2.1f") axis(1)) ylabel(-1(0.25)0, format("%3.2f") axis(2)) ytitle("", axis(2))												/// y-axis options
	xlabel(0(0.1)1, format("%2.1f")) xtitle("{&lambda}")																									// x-axis options
// save course3_simulation, replace
