************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* GMM Techniques for Dynamic Simultaneous Panel Data Models
************************************************************************

/*
This do-file generates the graphs for the iterated GMM estimator in the section "System GMM".

See the presentation slides for details on the data-generating process for the simulated data.
See the Stata help files for details on command syntax and functionality.

Note: The default graph color style changed in Stata 18.
With earlier versions, this do-file does not exactly replicate the appearance of the graphs in the presentation slides.
*/

*----------------------------------------------------------------------*
* required additional Stata package
net install xtdpdgmm, from("http://www.kripfganz.de/stata/")		// package to compute the GMM estimators

*----------------------------------------------------------------------*
* general set up
clear all
set seed 20230717							// random-number seed
local lambda = 0.2							// coefficient of the lagged dependent variable
local N = 60								// number of cross-sectional observations
local T = 5									// number of time-series observations
local obs = `N' * `T'
set obs `obs'								// number of observations
egen byte id = seq(), from(1) block(`T')	// unit identifier
egen byte t = seq(), from(1) to(`T')		// time identifier
xtset id t									// declare data set to be panel data

*----------------------------------------------------------------------*
* data generation
gen a = mod(id - 1, 3) - 1					// unit-specific error component
gen y = a / (1 - `lambda') + rnormal() / sqrt(1 - (`lambda')^2) if t == 1		// outcome variable (initial observations)
replace y = `lambda' * L.y + a + rnormal() if t > 1								// outcome variable (recursive process for subsequent observations)

*----------------------------------------------------------------------*
* iterated GMM estimation
foreach W in unadjusted independent separate {		// loop over 3 different initial weighting matrices
	gen b_`W' = .
	xtdpdgmm y L.y, gmmiv(y, lagrange(2 .) model(diff)) gmmiv(y, difference lagrange(1 1) model(level)) wmatrix(`W') igmm		// iterated Blundell-Bond estimator (to determine the number of iteration steps)
	local steps = e(steps)
	forvalues j = 1 / `steps' {				// loop over all steps for the iterated estimator
		quietly xtdpdgmm y L.y, gmmiv(y, lagrange(2 .) model(diff)) gmmiv(y, difference lagrange(1 1) model(level)) wmatrix(`W') igmm igmmiterate(`j')		// j-step Blundell-Bond estimator
		replace b_`W' = _b[L.y] in `j'
	}
}

*----------------------------------------------------------------------*
* graph
gen j = _n in 1 / 20

twoway line b_unadjusted j || line b_independent j || line b_separate j || , 				/// graph specification
	legend(label(1 "W (2000)") label(2 "BBW (2001)") label(3 "AB (1995), BB (1998)"))		/// graph options
	ylabel(0.15(0.01)0.25, format(%3.2f))													/// y-axis options
	xlabel(5(5)20) xscale(range(1/20)) xtitle("iteration step")								// x-axis options
