************************************************************************
* Advanced Dynamic Panel Data Methods
* Dr Sebastian Kripfganz
* University of Exeter Business School, Department of Economics
*
* Universidad de Salamanca
* July 17-19, 2023
*
* GMM Techniques for Dynamic Simultaneous Panel Data Models
************************************************************************

/*
This do-file generates the simulated estimator distributions in the section "Difference GMM".

See the presentation slides for details on the data-generating process for the simulated data.
See the Stata help files for details on command syntax and functionality.

Note: The default graph color style changed in Stata 18.
With earlier versions, this do-file does not exactly replicate the appearance of the graphs in the presentation slides.
*/

*----------------------------------------------------------------------*
* required additional Stata packages
net install xtdpdqml, from("http://www.kripfganz.de/stata/")		// package to compute the QML estimator
net install xtdpdgmm, from("http://www.kripfganz.de/stata/")		// package to compute the GMM estimators

*----------------------------------------------------------------------*
* general set up
clear all
set seed 20230717							// random-number seed
local lambda = 0.2							// coefficient of the lagged dependent variable
local SDa = 1								// standard deviation of the unit-specific error component

*----------------------------------------------------------------------*
* simulation program
program define mysimul, rclass
	local T = `1'							// number of time-series observations
	local Tall = `T' + 10					// number of time-series observations (including "burn-in" periods)
	local N = 3000 / `T'					// number of cross-sectional observations
	local lambda = `2'						// coefficient of the lagged dependent variable
	local SDa = `3'							// standard deviation of the unit-specific error component

	drop _all
	local obs = `N' * (`Tall')
	set obs `obs'									// number of observations (including "burn-in" periods)
	egen byte id = seq(), from(1) block(`Tall')		// unit identifier
	egen byte t = seq(), from(1) to(`Tall')			// time identifier
	xtset id t										// declare data set to be panel data

	gen a = `SDa' * mod(id - 1, 3) - 1				// unit-specific error component
	gen e = rnormal()								// idiosyncratic error component
	gen y = a / (1 - `lambda') + e / sqrt(1 - (`lambda')^2) if t == 1		// outcome variable (initial observations drawn from the stationary distribution, conditional on x and a)
	replace y = `lambda' * L.y + a + e if t > 1								// outcome variable (recursive process for subsequent observations)
	drop if t <= `Tall' - `T'						// drop "burn-in" periods

	xtdpdqml y, noconstant					// QML estimator
	local b_QML = _b[L.y]
	ivregress 2sls D.y (LD.y = L2.y), noconstant		// Anderson-Hsiao IV estimator
	local b_AH = _b[LD.y]
	xtdpdgmm y L.y, model(difference) gmmiv(y, lagrange(2 .)) noconstant				// Arellano-Bond one-step GMM estimator
	local b_AB = _b[L.y]
	xtdpdgmm y L.y, model(difference) gmmiv(y, lagrange(2 .)) collapse noconstant		// Arellano-Bond one-step GMM estimator (collapsed)
	local b_AB_col = _b[L.y]
	xtdpdgmm y L.y, model(difference) gmmiv(y, lagrange(2 4)) noconstant				// Arellano-Bond one-step GMM estimator (curtailed)
	local b_AB_cur = _b[L.y]
	xtdpdgmm y L.y, model(difference) gmmiv(y, lagrange(2 4)) collapse noconstant		// Arellano-Bond one-step GMM estimator (curtailed/collapsed)
	local b_AB_cc = _b[L.y]

	ereturn clear
	mata: mata drop xtdpdgmm_opt_1 xtdpdgmm_opt_2 xtdpdgmm_opt_3 xtdpdgmm_opt_4		// free up memory from some large Mata objects
	return scalar b_QML = `b_QML'
	return scalar b_AH = `b_AH'
	return scalar b_AB = `b_AB'
	return scalar b_AB_col = `b_AB_col'
	return scalar b_AB_cur = `b_AB_cur'
	return scalar b_AB_cc = `b_AB_cc'
end

*----------------------------------------------------------------------*
* simulation with T = 10 (and N = 300)
simulate , reps(1001) dots(10) : mysimul 10 `lambda' `SDa'		// 1001 replications
summarize														// summary statistics
save course4_simulation, replace

twoway kdensity b_QML || kdensity b_AH || kdensity b_AB || kdensity b_AB_col || kdensity b_AB_cur || kdensity b_AB_cc || ,										/// graph specification
	legend(label(1 "QML") label(2 "AH (1981)") label(3 "AB (1991)") label(4 "AB (1991), col.") label(5 "AB (1991), cur.") label(6 "AB (1991), cur./col."))		/// graph options
	ylabel(0(3)18, nolabel noticks) yscale(off) ytitle("")																										/// y-axis options
	xline(`lambda') xlabel(0(0.1)1.2, format("%2.1f")) xmlabel(`lambda' "{&lambda}", labgap(*10)) xtitle("")													// x-axis options
